import m from 'mithril';
import { Widget } from 'components/widgets/widget';
import { Pipe } from 'utilities/pipe';

require('./gauge-linear.css');

export class GaugeLinear extends Widget {
	private gaugeValue: Pipe<number>;
	private gaugeMaxValue: number;
	private bars: SVGRectElement[] = [];
	private valueElement: HTMLElement;
	
	constructor(vnode: any) {
		super(vnode);
		this.gaugeValue = vnode.attrs.gaugeValue || new Pipe(0.0);
		this.gaugeMaxValue = vnode.attrs.gaugeMaxValue || 1.0;
		this.gaugeValue.onChange(() => this.updateGauge());
	}
	
	view(vnode: m.Vnode<{}, {}>): m.Children {
		return <div class="widget gauge-linear" style={'flex: ' + this.widgetWidth}>
			<p><span class="value"></span><span class="unit">W</span></p>
			<svg>
			</svg>
		</div>;
	}
	
	createSvg(svgElement: SVGElement) {
		let w = Math.round(svgElement.clientWidth);
		let h = Math.round(svgElement.clientHeight);
		
		svgElement.setAttribute('viewBox', "0 0 "+w+" "+h);

		let bg = document.createElementNS('http://www.w3.org/2000/svg', 'rect');
		bg.setAttribute('width', w.toString());
		bg.setAttribute('height', h.toString());
		bg.classList.add('gauge-linear-bg');
		svgElement.append(bg);
		
		let barHeight = 15;
		const barGap = barHeight * 0.2;
		let numBars = Math.round(h / (barHeight + barGap));
		
		let bh = (h - (numBars+1)*barGap) / numBars;
		barHeight = Math.round(bh);
		
		for(let barIdx = 0; barIdx < numBars; ++barIdx) {
			let bar = document.createElementNS('http://www.w3.org/2000/svg', 'rect');
			bar.setAttribute('width', (w - barGap * 2).toString());
			bar.setAttribute('height', barHeight.toString());
			bar.classList.add('gauge-linear-bar');
			bar.classList.toggle('lit', false);
			
			bar.setAttribute('x', barGap.toString());
			bar.setAttribute('y', (h - barIdx * (bh+barGap) - barGap - barHeight).toString());
			
			svgElement.append(bar);
			
			this.bars.push(bar);
		}
	}
	
	updateGauge() {
		let value = Math.max(0.0, Math.min(this.gaugeMaxValue, this.gaugeValue.get()));
		this.valueElement.innerText = value.toFixed(0);
		
		let ratio = value / this.gaugeMaxValue;
		let numBars = this.bars.length;
		let litBars = Math.round(ratio * numBars);
		for(let barIdx = 0; barIdx < numBars; ++barIdx) {
			let bar = this.bars[barIdx];
			bar.classList.toggle('lit', barIdx < litBars);
		}
	}
	
	oncreate(vnode: any) {
		let svgElement: SVGElement = vnode.dom.querySelector('svg');
		this.createSvg(svgElement);
		
		this.valueElement = vnode.dom.querySelector('span.value');
		
		this.updateGauge();
	}
}
