import m from 'mithril';
import { Page } from 'components/page';
import { MonitorApi, Status } from 'monitor-api';
import { Pipe } from 'utilities/pipe';

import { Clock } from 'components/widgets/clock';
import { GaugeLinear } from 'components/widgets/gauge-linear';

require('./dashboard-page.css');

export class DashboardPage extends Page {
	status: Status = null;
	autoRefresh = true;
	
	private power = new Pipe(0.0);
	
	oninit() {
		this.status = MonitorApi.get().getStatus();
		this.refresh();
	}
	
	onbeforeremove(vnode: m.Vnode<{}, {}>) {
    	this.autoRefresh = false;
	}
	
	async refresh() {
		let newStatus = await MonitorApi.get().fetchStatus();
		if(this.status == null)
			m.redraw();
		this.status = newStatus;
		
		this.power.set(this.status.batteryVoltage * this.status.motorCurrent);
		
		if(this.autoRefresh)
			setTimeout(() => { if(this.autoRefresh) this.refresh(); }, 500);
	}
	
	view() {
		return this.status
			? <div class="dashboard-page">
				<div class="widgets-row">
					<Clock/>
				</div>
				<div class="widgets-row" style="height: 40%">
					<GaugeLinear widgetWidth={0.2} gaugeValue={this.power} gaugeMaxValue={999.0} />
				</div>
			</div>
			: <p>Chargement...</p>;
	}
}
